{--------------------------------------------------------------------------------------------------}
{
        Component :     TCoolProgBar
        Author    :     Sebastien Costa
        Date      :     03/11/2002

        Version   :     1.0

        Home Page :     http://web.tiscali.it/sebastien/delphi
        E-Mail    :     sebastien@tiscalinet.it

  Send me an E-mail if you have any comments, questions or bug report.
  If you find this component useful, I would enjoy hearing about it!
  Send me money for beer!!! Thank you!!!
  
  Legal issues:
  This component is freeware. I will not responsible for any damage
  that occurs when you use this component. Although it has been
  tested and works fine. USE IT AT YOUR OWN RISK !
  Feels free to use, modify, and redistribute it as long as this notice
  didn't removed or altered.

}
{--------------------------------------------------------------------------------------------------}
unit CoolProgBar;

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, Graphics, ExtCtrls;

type

  TSCBarDirection = (bdLeftToRight, bdBottomToTop);
  TSCBorderStyle  = (bsNone, bsLowered, bsRaised);
  TSCBorderWidth  = (bwThin, bwLarge);

  {-------------------------------------}
  TSCCustomCoolProgBar = class(TGraphicControl)
  private
    { Private declarations }
    FBlock        : TBitmap;
    FBlockOn      : TBitmap;
    FBlockOff     : TBitmap;
    FMemBmp       : TBitmap;

    FMax          : Integer;
    FMin          : Integer;
    FPos          : Integer;

    FBarDirection : TSCBarDirection;
    FBorderStyle  : TSCBorderStyle;
    FBorderWidth  : TSCBorderWidth;
    FBorder       : Integer;

    FVisible      : boolean;

    FBackColor    : TColor;

    procedure SplitImages;

    procedure SetBlock(const Value: TBitmap);
    procedure SetMax(const Value: Integer);
    procedure SetMin(const Value: Integer);
    procedure SetPos(const Value: Integer);
    procedure SetBackColor(Value: TColor);
    procedure SetBarDirection(Value: TSCBarDirection);
    procedure SetBorderStyle(Value: TSCBorderStyle);
    procedure SetBorderWidth(Value: TSCBorderWidth);
    procedure SetVisible(const Value: boolean);

  protected
    { Protected declarations }
    procedure Loaded; override;
    procedure Paint; override;
    property BlockGlyph   : TBitmap read FBlock write SetBlock;
    property BackColor    : TColor  read FBackColor write SetBackColor default clBlack;
    property Max          : Integer read FMax   write SetMax    default 100;
    property Min          : Integer read FMin   write SetMin    default 0;
    property Pos          : Integer read FPos   write SetPos    default 0;
    property BarDirection : TSCBarDirection read FBarDirection write SetBarDirection default bdLeftToRight;
    property BorderStyle  : TSCBorderStyle  read FBorderStyle  write SetBorderStyle  default bsLowered;
    property BorderWidth  : TSCBorderWidth  read FBorderWidth  write SetBorderWidth  default bwThin;
    property Visible      : boolean read FVisible write SetVisible default true;

  public
    { Public declarations }
    constructor Create( AOwner: TComponent ); override;
    destructor Destroy; override;

  published
    { Published declarations }

  end;

  {-------------------------------------}
  TSCCoolProgBar = class(TSCCustomCoolProgBar)
  private
    { Private declarations }
  protected
    { Protected declarations }
  public
    { Public declarations }
  published
    { Published declarations }
    property BlockGlyph;
    property BackColor;
    property Max;
    property Min;
    property Pos;
    property BarDirection;
    property BorderStyle;
    property BorderWidth;
    property Visible;
  end;

procedure Register;

{--------------------------------------------------------------------------------------------------}
{--------------------------------------------------------------------------------------------------}
{--------------------------------------------------------------------------------------------------}
implementation

{--------------------------------------------------------------------------------------------------}
procedure Register;
begin
  RegisterComponents('Seb', [TSCCoolProgBar]);
end;

{--------------------------------------------------------------------------------------------------}
constructor TSCCustomCoolProgBar.Create;
begin
  inherited;
  FBlock        := TBitmap.Create;
  FBlockOn      := TBitmap.Create;
  FBlockOff     := TBitmap.Create;
  FMemBmp       := TBitmap.Create;
  FMax          := 100;
  FMin          := 0;
  FPos          := 0;
  FBarDirection := bdLeftToRight;
  FBorderWidth  := bwThin;
  FBorderStyle  := bsLowered;
  FBorder       := 2;
  FBackColor    := clBlack;
  Visible       := true;

  ControlStyle  := ControlStyle + [csFramed, csOpaque]; { reduce flicker on component redraw }
end;

{--------------------------------------------------------------------------------------------------}
destructor TSCCustomCoolProgBar.Destroy;
begin
  FBlock.Free;
  FBlockOn.Free;
  FBlockOff.Free;
  FMemBmp.Free;
  inherited;
end;

{--------------------------------------------------------------------------------------------------}
procedure TSCCustomCoolProgBar.Paint;
var
  num_lines   : Integer;
  num_linesOn : Integer;
  i           : Integer;
  temp        : Integer;
  tmprect     : TRect;
  posx        : Integer;
  posy        : Integer;
begin
  if (Not FVisible) And (Not (csDesigning in ComponentState)) then
    exit;

  { prepare off-screen drawing }
  FMemBmp.Width  := Width;
  FMemBmp.Height := Height;

  { fill rectangle with background color }
  FMemBmp.canvas.brush.color := FBackColor;
  tmprect := Rect(0,0,width,height);
  FMemBmp.canvas.FillRect(tmprect);

  { draw border }
  if (FBorderStyle <> bsNone) then begin
    if (FBorderStyle = bsRaised)
      then
        Frame3D(FMemBmp.canvas, tmprect, clWhite, clGray,  FBorder)
      else
        Frame3D(FMemBmp.canvas, tmprect, clGray,  clWhite, FBorder);
  end;

  { if a bitmap is defined we can draw the bar :-) }
  if (FBlockOn.Width > 0) And (FBlockOn.Height > 0) then
  begin
    { test FMin and FMax are in correct order }
    if (FMin > FMax) then
    begin
      temp := FMax;
      FMax := FMin;
      FMin := temp;
    end;

    if FBarDirection = bdLeftToRight Then
    begin
      posy      := (ClientHeight-FBorder*2 - FBlockOn.Height) DIV 2 + FBorder;
      num_lines := (ClientWidth-FBorder*2) DIV FBlockOn.Width;
      posx      := ((ClientWidth-FBorder*2) MOD FBlockOn.Width) DIV 2 + FBorder;
    end
    else
    begin
      posx      := (ClientWidth-FBorder*2 - FBlockOn.Width) DIV 2 + FBorder;
      num_lines := (ClientHeight-FBorder*2) DIV FBlockOn.Height;
      posy      := ((ClientHeight-FBorder*2) MOD FBlockOn.Height) DIV 2 + FBorder;
    end;

    if (FPos = 0) or (FMax = FMin) then
      num_linesOn := 0
    else
      num_linesOn := (FPos * num_lines) DIV (FMax - FMin);

    if FBarDirection = bdLeftToRight Then
    begin
      for i:= 0 to (num_linesOn-1) do
        FMemBmp.Canvas.Draw((i*FBlockOn.Width)+posx,posy,FBlockOn);
      for i:= num_linesOn to (num_lines-1) do
        FMemBmp.Canvas.Draw((i*FBlockOn.Width)+posx,posy,FBlockOff);
    end
    else
    begin
      for i:= 0 to (num_linesOn-1) do
        FMemBmp.Canvas.Draw(posx, ClientHeight-posy-((i+1)*FBlockOn.Height),FBlockOn);
      for i:= num_linesOn to (num_lines-1) do
        FMemBmp.Canvas.Draw(posx, ClientHeight-posy-((i+1)*FBlockOn.Height),FBlockOff);
    end

  end;

  Canvas.Draw(0, 0, FMemBmp);
end;

{--------------------------------------------------------------------------------------------------}
procedure TSCCustomCoolProgBar.Loaded;
begin
  if Assigned(FBlock) then
    SplitImages;
end;

{--------------------------------------------------------------------------------------------------}
procedure TSCCustomCoolProgBar.SetBlock(const Value: TBitmap);
begin
  FBlock.Assign(Value);
  SplitImages;
  Repaint;
end;

{--------------------------------------------------------------------------------------------------}
procedure TSCCustomCoolProgBar.SetMax(const Value: Integer);
begin
  if FMax <> Value Then
  begin
    FMax := Value;
    Repaint;
  end;
end;

{--------------------------------------------------------------------------------------------------}
procedure TSCCustomCoolProgBar.SetMin(const Value: Integer);
begin
  if FMin <> Value Then
  begin
    FMin := Value;
    Repaint;
  end;
end;

{--------------------------------------------------------------------------------------------------}
procedure TSCCustomCoolProgBar.SetPos(const Value: Integer);
begin
  if FPos <> Value Then
  begin
    if Value > FMax then
      FPos := FMax
    else
      if Value < FMin then
        FPos := FMin
      else
        FPos := Value;
    Repaint;
  end;
end;

{--------------------------------------------------------------------------------------------------}
procedure TSCCustomCoolProgBar.SetVisible(const Value: boolean);
begin
  if FVisible <> Value Then
  begin
    FVisible := Value;
    Repaint;
  end;
end;

{--------------------------------------------------------------------------------------------------}
procedure TSCCustomCoolProgBar.SetBarDirection(Value: TSCBarDirection);
begin
  if (FBarDirection <> Value) then
  begin
    FBarDirection := Value;
    SplitImages;        { we must regenerate our blocks }
    Repaint;
  end;
end;

{--------------------------------------------------------------------------------------------------}
procedure TSCCustomCoolProgBar.SetBorderStyle(Value: TSCBorderStyle);
begin
  if (FBorderStyle <> Value) then
  begin
    FBorderStyle := Value;
    if (FBorderStyle = bsNone) then
      FBorder := 0
    else
    begin
      if (FBorderWidth = bwThin)
        then FBorder := 1
        else FBorder := 2;
    end;
    Repaint;
  end;
end;

{--------------------------------------------------------------------------------------------------}
procedure TSCCustomCoolProgBar.SetBorderWidth(Value: TSCBorderWidth);
begin
  if (FBorderWidth <> Value) then
  begin
    FBorderWidth := Value;

    if (FBorderWidth = bwThin)
      then FBorder := 1
      else FBorder := 2;

    Repaint;
  end;
end;

{--------------------------------------------------------------------------------------------------}
procedure TSCCustomCoolProgBar.SetBackColor(Value: TColor);
begin
  if Value <> FBackColor then begin
    FBackColor := Value;
    Repaint;
  end;
end;

{--------------------------------------------------------------------------------------------------}
procedure TSCCustomCoolProgBar.SplitImages;
var
  dest, src : TRect;
begin
  dest.Left   := 0;
  dest.Top    := 0;

  dest.Right  := (FBlock.Width DIV 2);
  dest.Bottom := FBlock.Height;
  src.Top     := 0;
  src.Bottom  := FBlock.Height;

  src.Left    := 0;
  src.Right   := (FBlock.Width DIV 2);
  FBlockOn.Width  := FBlock.Width DIV 2;
  FBlockOn.Height := FBlock.Height;
  FBlockOn.Canvas.CopyRect(dest, FBlock.Canvas, src);

  src.Left    := (FBlock.Width DIV 2);
  src.Right   := FBlock.Width;
  FBlockOff.Width  := FBlock.Width DIV 2;
  FBlockOff.Height := FBlock.Height;
  FBlockOff.Canvas.CopyRect(dest, FBlock.Canvas, src);
end;

end.
